@@ -349,7 +349,7 @@ setMethod("as.period", signature(x = "character"), function(x, ...) {
 setMethod("as.period", signature(x = "numeric"), function(x, unit = "second", ...) {
   x <- as.numeric(x)
   if (missing(unit)) unit <- "second"
-  unit <- standardise_date_names(unit)
+  unit <- standardise_date_names(unit[[1]])
   f <- get(paste(unit, "s", sep = ""),
            envir = asNamespace("lubridate"),
            mode = "function", inherits = FALSE)
@@ -484,8 +484,8 @@ setMethod("as.period", signature(x = "Duration"), function(x, unit = NULL, ...)
   remainder <- abs(span)
   newper <- period(second = rep(0, length(x)))
 
-  slot(newper, "year") <- remainder %/% (3600 * 24 * 365)
-  remainder <- remainder %% (3600 * 24 * 365)
+  slot(newper, "year") <- remainder %/% average_durations[["year"]]
+  remainder <- remainder %% average_durations[["year"]]
 
   slot(newper, "day") <- remainder %/% (3600 * 24)
   remainder <- remainder %% (3600 * 24)
@@ -524,7 +524,6 @@ setMethod("as.period", signature("logical"), function(x, unit = NULL, ...) {
   as.period(as.numeric(x), unit, ...)
 })
 
-#' @export
 setGeneric("as.difftime")
 
 #' @export
@@ -604,6 +603,7 @@ setMethod("as.character", signature(x = "Interval"), function(x, ...) {
 #'      a more intuitive conversion (see examples)
 #'   \item Both functions provide a default origin argument for numeric
 #'      vectors.
+#'   \item Both functions will generate NAs for invalid date format. A warning message will provide a count of the elements that were not converted
 #'   \item `as_datetime()` defaults to using UTC.
 #' }
 #'
@@ -626,6 +626,9 @@ setMethod("as.character", signature(x = "Interval"), function(x, ...) {
 #' c(as_date(dt_europe), as.Date(dt_europe))
 #' ## need not supply origin
 #' as_date(10)
+#' ## Will replace invalid date format with NA
+#' dt_wrong <- c("2009-09-29", "2012-11-29", "2015-29-12")
+#' as_date(dt_wrong)
 #' @export
 setGeneric(name = "as_date",
            def = function(x, ...) standardGeneric("as_date"))
@@ -670,10 +673,14 @@ setMethod(f = "as_date", signature = "numeric",
 #' @export
 setMethod("as_date", "character",
           function(x, tz = NULL, format = NULL) {
+            if (!is.null(tz)) {
+              warning("`tz` argument is ignored by `as_date()`", call. = FALSE)
+            }
+
             if (is.null(format))
-              as_date(as_datetime(x, tz = "UTC"))
+              as_date(.parse_iso_dt(x, tz = "UTC"))
             else
-              as_date(strptime(x, format, tz))
+              as_date(strptime(x, format, tz = "UTC"))
           })
 
 #' @rdname as_date
@@ -705,7 +712,7 @@ setMethod("as_datetime", "character",
             if (is.null(format))
               .parse_iso_dt(x, tz)
             else
-              strptime(x, format = format, tz = tz)
+              as.POSIXct(strptime(x, format = format, tz = tz))
           })
 
 #' @rdname as_date
